@@ -1,5 +1,6 @@
 from .bfp import BFP
 from .channel_mapper import ChannelMapper
+from .ct_resnet_neck import CTResNetNeck
 from .dilated_encoder import DilatedEncoder
 from .fpg import FPG
 from .fpn import FPN
@@ -13,5 +14,5 @@ from .yolo_neck import YOLOV3Neck
 
 __all__ = [
     'FPN', 'BFP', 'ChannelMapper', 'HRFPN', 'NASFPN', 'FPN_CARAFE', 'PAFPN',
-    'NASFCOS_FPN', 'RFP', 'YOLOV3Neck', 'FPG', 'DilatedEncoder'
+    'NASFCOS_FPN', 'RFP', 'YOLOV3Neck', 'FPG', 'DilatedEncoder', 'CTResNetNeck'
 ]
